using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.Azure.WebJobs.Host;
using Microsoft.Extensions.Logging;
using Microsoft.ApplicationInsights;
using Newtonsoft.Json;
using System.Collections.Generic;

namespace Karlex.ErrorHandling.Functions
{
    public static class LogInformation
    {
        [FunctionName("LogInformation")]
        public static async Task<HttpResponseMessage> Run([HttpTrigger(AuthorizationLevel.Function,  "post", Route = null)]HttpRequestMessage req, TraceWriter log, ILogger logger)
        {
            try
            {
                TelemetryClient client = new TelemetryClient();

                logger.LogInformation("LogInformationOrError started ");
                LogRequestBody data = await req.Content.ReadAsAsync<LogRequestBody>();
            
                client.Context.GlobalProperties.Add("Correlation Id", data.CorrelationId);
                client.Context.GlobalProperties.Add("Environment Id", data.EnvironmentId);
                client.Context.GlobalProperties.Add("Flow Name", data.FlowName);
                client.Context.GlobalProperties.Add("Flow Url", data.FlowUrl);
                client.Context.GlobalProperties.Add("Error Status", data.ErrorStatus);
                client.Context.GlobalProperties.Add("Error Code", data.ErrorCode);
                client.Context.GlobalProperties.Add("Error Description", data.ErrorDescription);
                client.Context.GlobalProperties.Add("Tracking Id", data.TrackingId);

                client.TrackEvent("Power Automate: " + data.ErrorDescription);

                return req.CreateResponse(HttpStatusCode.OK, "Information logged with global properties");
            }
            catch (Exception ex)
            {
                logger.LogError("Failed to run loggin, message: ", ex.Message);
                return req.CreateResponse(HttpStatusCode.BadRequest, "Error in function " + ex.Message);
            }
        }
    }
}
